# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-21 07:52
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('editorial_staff', '0004_auto_20170921_0032'),
    ]

    operations = [
        migrations.AddField(
            model_name='staffer',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=datetime.datetime(2017, 7, 1, 7, 0, tzinfo=utc)),
            preserve_default=False,
        ),
    ]



# Imports from editorial-staff.  # NOQA
from editorial_staff.data_providers import SlackProvider
from editorial_staff.models import Staffer


SLACK = SlackProvider()


def load_staff_from_slack():
    """TK."""
    all_slack_users = SLACK.get_all_staffers()

    active_staff = [
        _ for _ in all_slack_users
        if _.get('deleted', False) is False
        and _.get('is_restricted', False) is False
        and _.get('is_ultra_restricted', False) is False
        and _.get('is_bot', False) is False
        and _.get('real_name', '').lower() != 'slackbot'
    ]

    added_staffers = []
    for slack_user in active_staff:
        formatted_staffer = SLACK.format_staffer(slack_user)
        new_staffer = Staffer(**formatted_staffer)
        new_staffer.save()

        added_staffers.append(new_staffer)

    return added_staffers



# Imports from Django.  # NOQA
from django.conf.urls import include, url  # NOQA


# Imports from editorial-staff.
from editorial_staff.views import admin_views, api_views  # NOQA


app_name = 'editorial_staff'


# Admin-site views.
urlpatterns = [
    url(
        r'^$',
        admin_views.Staffers.as_view(),
        name='staffer-list'
    ),
    url(
        r'^create/$',
        admin_views.StafferCreate.as_view(),
        name='staffer-create'
    ),
    # TODO(ajv): Add staffer detail view.
    url(
        r'^(?P<pk>[0-9]+)/$',
        admin_views.StafferDetail.as_view(),
        name='staffer-detail'
    ),
    url(
        r'^(?P<pk>[0-9]+)/edit/$',
        admin_views.StafferUpdate.as_view(),
        name='staffer-edit'
    ),

    # API views.
    url(r'^api/', include(
        [
            url(r'^$', api_views.RootAPIView.as_view(), name='root'),
            url(r'^staff/', include([
                url(
                    r'^$',
                    api_views.StaffList.as_view(),
                    name='staffer-list'
                ),
                url(
                    r'^rescrape/(?P<email>.+)/$',
                    api_views.StaffRescrape.as_view(),
                    name='staffer-rescrape'
                ),
                url(
                    r'^(?P<email>.+)/$',
                    api_views.StaffFetch.as_view(),
                    name='staffer-detail'
                ),
            ])),
            url(r'^hub/', include([
                url(
                    r'^$',
                    api_views.HubList.as_view()
                ),
                url(
                    r'^(?P<slug>.+)/$',
                    api_views.HubFetch.as_view()
                ),
            ])),
            url(r'^vertical/', include([
                url(
                    r'^$',
                    api_views.VerticalList.as_view()
                ),
                url(
                    r'^(?P<slug>.+)/$',
                    api_views.VerticalFetch.as_view()
                ),
            ])),
        ],
        namespace='api'
    )),
]



# Imports from Django.  # NOQA
from django.db import models


# Imports from editorial-staff.
from editorial_staff.utils import get_random_anonymous_animal


# Imports from other dependencies.
from colorfield.fields import ColorField


class Vertical(models.Model):
    """Model for top-level organizational entities."""
    name = models.CharField(max_length=100)
    slug = models.SlugField(max_length=100)

    class Meta:  # noqa
        ordering = ['name']

    def __str__(self):
        return self.name

    def as_object(self):
        return {
            'name': self.name,
            'slug': self.slug,
        }


class HubManager(models.Manager):
    def get_queryset(self):
        return super(
            HubManager,
            self
        ).get_queryset().select_related('vertical')


class Hub(models.Model):
    """Model for lower-level organizational entities."""
    name = models.CharField(max_length=100)
    slug = models.SlugField(max_length=100)
    vertical = models.ForeignKey(Vertical, on_delete=models.CASCADE)
    color = ColorField(default='#0185D3')

    objects = HubManager()

    class Meta:  # noqa
        ordering = ['name']

    def __str__(self):
        return self.name

    def as_object(self):
        return {
            'name': self.name,
            'color': self.color,
            'slug': self.slug,
            'vertical': {
                'name': self.vertical.name,
                'slug': self.vertical.slug,
            }
        }


class Staffer(models.Model):
    """Model for individual staffers."""
    email = models.EmailField(unique=True)
    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    image_url = models.URLField(
        blank=True,
        null=True,
        help_text='Usually auto-populated from Slack.'
    )
    active = models.BooleanField(default=True)

    created = models.DateTimeField(
        auto_now_add=True,
        editable=False,
    )

    class Meta:  # noqa
        ordering = ['last_name', 'first_name']

    def __str__(self):
        return self.render_full_name()

    def save(self, *args, **kwargs):
        if not self.last_name and not self.first_name:
            self.first_name = 'anonymous'
            self.last_name = get_random_anonymous_animal()
        super(Staffer, self).save(*args, **kwargs)

    def render_full_name(self):
        if self.last_name:
            if self.first_name:
                return '{} {}'.format(self.first_name, self.last_name)

            return self.last_name

        return ''

    def render_formatted_name(self):
        # TK: More error-proof formatting for people with the same
        # surname.
        return self.last_name

    def as_object(self):
        return {
            'firstName': self.first_name,
            'lastName': self.last_name,
            'fullName': self.render_full_name(),
            'formattedName': self.render_formatted_name(),
            'email': self.email,
            'imageURL': self.image_url,
            'active': self.active,
        }



# Imports from django.  # NOQA
from django.db.utils import ProgrammingError
from django import forms
from django.utils.safestring import mark_safe


# Imports from editorial-staff.
from editorial_staff.models import Vertical


class VerticalWidget(forms.widgets.Select):
    """A <select> widget that uses our staff API verticals as choices"""
    def __init__(self, *args, **kwargs):
        try:
            kwargs['choices'] = Vertical.objects.values_list('slug', 'name')
            return super(VerticalWidget, self).__init__(*args, **kwargs)
        except ProgrammingError:
            # For cases where the staff_vertical table isn't ready yet
            kwargs['choices'] = list()
            return super(VerticalWidget, self).__init__(*args, **kwargs)


class StafferSelectWidget(forms.Textarea):
    class Media:  # NOQA
        css = {
            'all': (
                'staff/staff-select-widget/css/selectize.css',
                'staff/staff-select-widget/css/staffer-select.css',
            )
        }
        js = (
            'staff/staff-select-widget/js/selectize.js',
            'staff/staff-select-widget/js/underscore.min.js',
            'staff/staff-select-widget/js/staffer-select.js',
        )

    def __init__(self, attrs=None, single_value=False):
        self.single_value = False
        if single_value:
            self.single_value = True
        super(StafferSelectWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        original_html = super(StafferSelectWidget, self).render(
            name,
            value,
            attrs,
        )

        if self.single_value:
            selection_class = 'single-value'
            placeholder_text = 'Select a staffer...'
        else:
            selection_class = 'multi-value'
            placeholder_text = 'Choose one or more staffers...'

        total_html = """
        <div class="staffer-select-outer """ + selection_class + """ ">
            <div class="original-html" style="display: none">
            """ + original_html + """
            </div>
            <select
                multiple
                class="staffer-select"
                placeholder='""" + placeholder_text + """'>
            </select>
        </div>
        """

        return mark_safe(total_html)



# Imports from Django.  # NOQA
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic.base import RedirectView
from django.views.generic import View
from django.views.generic.edit import CreateView, UpdateView  # NOQA
from django.shortcuts import render_to_response  # NOQA
from django.urls import reverse_lazy


# Imports from editorial-staff.
from editorial_staff.models import Staffer  # NOQA


class Staffers(LoginRequiredMixin, View):
    def get(self, request):
        staff = Staffer.objects.all()

        latest_additions = Staffer.objects.order_by('-created')[:10]

        return render_to_response(
            'editorial_staff/staffer_list.html',
            {
                'latest_additions': latest_additions,
                'staff': staff
            }
        )


class StafferCreate(LoginRequiredMixin, CreateView):
    template_name = 'editorial_staff/staffer_create.html'
    model = Staffer
    fields = ['email', 'first_name', 'last_name', 'active', 'image_url']

    def get_context_data(self, *args, **kwargs):
        """Add 'STAFF_EMAIL_DOMAIN' setting to context."""
        context = super(StafferCreate, self).get_context_data(*args, **kwargs)

        email_domain = getattr(settings, 'STAFF_EMAIL_DOMAIN', 'example.com')

        context.update({'mainEmailDomain': email_domain})

        return context

    def get_success_url(self):
        submit_action = self.request.POST.get('submit', '_save')

        if submit_action == '_continue':
            return reverse_lazy('editorial_staff:staffer-edit', kwargs={
                'pk': self.object.id,
            })

        return reverse_lazy('editorial_staff:staffer-list')


class StafferDetail(LoginRequiredMixin, RedirectView):
    permanent = False
    query_string = True
    pattern_name = 'editorial_staff:staffer-edit'


class StafferUpdate(LoginRequiredMixin, UpdateView):
    template_name = 'editorial_staff/staffer_edit.html'
    model = Staffer
    fields = ['email', 'first_name', 'last_name', 'active', 'image_url']

    def get_success_url(self):
        submit_action = self.request.POST.get('submit', '_save')

        if submit_action == '_continue':
            return reverse_lazy(
                'editorial_staff:staffer-edit',
                kwargs=self.kwargs
            )

        return reverse_lazy('editorial_staff:staffer-list')



# Imports from Django.  # NOQA
from django.contrib import admin


# Imports from editorial-staff.
from editorial_staff.models import Staffer, Hub, Vertical  # NOQA


admin.site.register(Staffer)
admin.site.register(Hub)
admin.site.register(Vertical)



# Imports from Django.  # NOQA
from django.conf import settings


# Imports from editorial-staff.
# from editorial_staff.utils import get_random_anonymous_animal


# Imports from other dependencies.
from slacker import Slacker


SLACK_CONNECTION = Slacker(getattr(settings, 'SLACK_TOKEN', ''))


class SlackProvider(object):
    def __init__(self):
        self.connection = SLACK_CONNECTION

    def get_all_staffers(self):
        return self.connection.users.list().body['members']

    def get_staffer(self, queried_email):
        """Return a user if found in Slack."""
        users = self.connection.users.list().body['members']
        for user in users:
            if user['profile'].get('email', None) == queried_email:
                return user
        return None

    def format_staffer(self, user_data, exclude_email=False):
        """TK."""
        staffer_formatted = {}

        profile = user_data['profile']

        if exclude_email is not True:
            staffer_formatted['email'] = profile.get('email')

        staffer_formatted['last_name'] = profile.get('last_name', '').strip()
        if len(staffer_formatted['last_name']) == 0:
            staffer_formatted['last_name'] = None

        staffer_formatted['first_name'] = profile.get('first_name', '').strip()
        if len(staffer_formatted['first_name']) == 0:
            staffer_formatted['first_name'] = None

        staffer_formatted['image_url'] = profile.get('image_72', '')

        return staffer_formatted



# Imports from Django.  # NOQA
from django.forms import (  # NOQA
    TextInput,
    DateInput,
    FileInput,
    CheckboxInput,
    # MultiWidget,
    ClearableFileInput,
    # Select,
    # RadioSelect,
    # CheckboxSelectMultiple,
    NumberInput,
    EmailInput,
    URLInput
)


# Imports from other dependencies.
from bootstrap3.forms import render_label
from bootstrap3.renderers import FieldRenderer
from bootstrap3.text import text_value


WIDGETS_WITH_UNDERLINE = (
    TextInput,
    DateInput,
    FileInput,
    ClearableFileInput,
    NumberInput,
    EmailInput,
    URLInput
)


class ImmaterialFieldRenderer(FieldRenderer):
    def post_widget_render(self, html):
        """Override adds helper class to checkboxes."""
        if isinstance(self.widget, CheckboxInput):
            return self.put_inside_label(
                '{}{}'.format(
                    html,
                    '<span class="ripple"></span><i class="helper"></i>'
                )
            )
        else:
            return super(ImmaterialFieldRenderer, self).post_widget_render(
                html
            )

    def add_label(self, html):
        """Override adds label _after_ input."""
        label = self.get_label()
        if label:
            if isinstance(self.widget, WIDGETS_WITH_UNDERLINE):
                html = '{}{}{}'.format(
                    html,
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    ),
                    '<i class="bar"></i>'
                )
            else:
                html = '{}{}{}'.format(
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    ),
                    html
                )
        return html

    def _render(self):
        """Override adds helptext (append_to_field) later than usual."""
        # See if we're not excluded
        if self.field.name in self.exclude.replace(' ', '').split(','):
            return ''
        # Hidden input requires no special treatment
        if self.field.is_hidden:
            return text_value(self.field)

        # Render the widget
        self.add_widget_attrs()
        html = self.field.as_widget(attrs=self.widget.attrs)

        self.restore_widget_attrs()

        # Start post render
        html = self.post_widget_render(html)

        html = self.wrap_widget(html)

        html = self.make_input_group(html)

        html = self.wrap_field(html)
        html = self.add_label(html)
        html = self.append_to_field(html)
        html = self.wrap_label_and_field(html)
        return html



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-21 19:39
from __future__ import unicode_literals

import colorfield.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('editorial_staff', '0002_staffer_active'),
    ]

    operations = [
        migrations.AlterField(
            model_name='hub',
            name='color',
            field=colorfield.fields.ColorField(default='#0185D3', max_length=10),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-27 20:28
from __future__ import unicode_literals

import colorfield.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Hub',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('slug', models.SlugField(max_length=100)),
                ('color', colorfield.fields.ColorField(default=b'#0185D3', max_length=10)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Staffer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('first_name', models.CharField(blank=True, max_length=50, null=True)),
                ('last_name', models.CharField(blank=True, max_length=50, null=True)),
                ('image_url', models.URLField(blank=True, null=True)),
            ],
            options={
                'ordering': ['last_name', 'first_name'],
            },
        ),
        migrations.CreateModel(
            name='Vertical',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('slug', models.SlugField(max_length=100)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='hub',
            name='vertical',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='editorial_staff.Vertical'),
        ),
    ]



# Imports from Django.  # NOQA
from django.conf import settings
from django.http import JsonResponse
from django.utils.module_loading import import_string
from django.views.generic import View


# Imports from editorial-staff.
from editorial_staff.models import Staffer, Hub, Vertical  # NOQA


PROVIDER_MODULE = getattr(settings, 'STAFF_DATA_PROVIDER', None)

DATA_PROVIDER = None
if PROVIDER_MODULE is not None:
    try:
        DataProviderClass = import_string(PROVIDER_MODULE)
        DATA_PROVIDER = DataProviderClass()
    except ImportError:
        pass


class RootAPIView(View):
    """Provides a base URL for staff-API requests from JS frontends."""
    def get(self, request):
        return JsonResponse({
            'status': 200,
            'message': 'You are at the root of the staff API.',
        })


class StaffList(View):
    def get(self, request):
        staff = [
            s.as_object() for s in Staffer.objects.all()
        ]
        return JsonResponse(staff, safe=False)


class StaffFetch(View):
    def get(self, request, email):
        """Return a user instance.

        If user not found, query slack and create user if email exists.
        """
        staffer = Staffer.objects.filter(email=email).first()
        if staffer:
            # TODO(ajv): This should return a response that includes, at its
            # top level:
            #   - 'status': 200 (normal execution)
            #   - 'staffer': { .. } (value of staffer.as_object())
            return JsonResponse(staffer.as_object(), safe=False)
        else:
            print('A')
            print(email)

            if DATA_PROVIDER is not None:
                staffer_raw = DATA_PROVIDER.get_staffer(email)

                if staffer_raw is not None:
                    staffer_fmt = DATA_PROVIDER.format_staffer(staffer_raw)

                    new_staffer = Staffer.objects.create(**staffer_fmt)
                    new_staffer.save()

                    # TODO(ajv): This should return a response that includes,
                    # at its top level:
                    #   - 'status': 201 (record created)
                    #   - 'staffer': { .. } (value of new_staffer.as_object())
                    return JsonResponse(new_staffer.as_object(), safe=False)

            return JsonResponse({
                'status': 404,
                'msg': 'No staff member found.',
            })


class StaffRescrape(View):
    def get(self, request, email):
        if DATA_PROVIDER is not None:
            staffer_raw = DATA_PROVIDER.get_staffer(email)

            if staffer_raw is not None:
                staffer_fmt = DATA_PROVIDER.format_staffer(staffer_raw)

                return JsonResponse({
                    'status': 200,
                    'staffer': staffer_fmt,
                    'details': staffer_raw,
                })
            return JsonResponse({
                'status': 404,
                'msg': 'Staffer not found.',
            })

        return JsonResponse({
            'status': 501,
            'msg': ' '.join([
                'Rescraping requires a valid option in',
                '`settings.STAFF_DATA_PROVIDER`.'
            ])
        })


class HubList(View):
    def get(self, request):
        hubs = [
            h.as_object() for h in Hub.objects.all()
        ]
        return JsonResponse(hubs, safe=False)


class HubFetch(View):
    def get(self, request, slug):
        hub = Hub.objects.filter(slug=slug).first()
        if hub:
            return JsonResponse(hub.as_object(), safe=False)
        return JsonResponse({'msg': 'No hub found.'})


class VerticalList(View):
    def get(self, request):
        verticals = [
            v.as_object() for v in Vertical.objects.all()
        ]
        return JsonResponse(verticals, safe=False)


class VerticalFetch(View):
    def get(self, request, slug):
        vertical = Vertical.objects.filter(slug=slug).first()
        if vertical:
            return JsonResponse(vertical.as_object(), safe=False)
        return JsonResponse({'msg': 'No vertical found.'})



# Imports from Django.  # NOQA
from django.db.models import Field


# Imports from editorial-staff.
from editorial_staff.validators import validate_vertical
from editorial_staff.widgets import VerticalWidget


class VerticalField(Field):
    def __init__(self, *args, **kwargs):
        validators = kwargs.get('validators', [])
        if validators is None:
            validators = []
        if validate_vertical not in validators:
            validators.append(validate_vertical)
        kwargs['validators'] = validators

        kwargs['max_length'] = 64
        return super(VerticalField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'CharField'

    def formfield(self, *args, **kwargs):
        kwargs['widget'] = kwargs.get('widget', VerticalWidget())
        return super(VerticalField, self).formfield(*args, **kwargs)



# Imports from Django.  # NOQA
from django.core.exceptions import ValidationError


# Imports from editorial-staff.
from editorial_staff.models import Vertical


def validate_vertical(slug):
    """Validate that the passed slug corresponds to an existing vertical

    :raises ValidationError: thrown if slug doesn't match
    """
    try:
        Vertical.objects.get(slug=slug)
    except Vertical.DoesNotExist:
        raise ValidationError('Vertical "%s" does not exist.' % slug)



# Imports from Django.  # NOQA
from django.apps import AppConfig


class EditorialStaffConfig(AppConfig):
    name = 'editorial_staff'
    verbose_name = 'Editorial staff'

    def ready(self):
        pass



# Imports from python.  # NOQA
import random


ANONYMOUS_ANIMALS = [
    'alligator', 'anteater', 'armadillo', 'auroch',
    'axolotl', 'badger', 'bat', 'beaver', 'buffalo',
    'camel', 'chameleon', 'cheetah', 'chipmunk',
    'chinchilla', 'chupacabra', 'cormorant', 'coyote', 'crow',
    'dingo', 'dinosaur', 'dolphin', 'duck', 'elephant',
    'ferret', 'fox', 'frog', 'giraffe', 'gopher', 'grizzly',
    'hedgehog', 'hippo', 'hyena', 'jackal', 'ibex',
    'ifrit', 'iguana', 'koala', 'kraken', 'lemur', 'leopard',
    'liger', 'llama', 'manatee', 'mink', 'monkey', 'narwhal',
    'nyan cat', 'orangutan', 'otter', 'panda', 'penguin',
    'platypus', 'python', 'pumpkin', 'quagga', 'rabbit',
    'raccoon', 'rhino', 'sheep', 'shrew', 'skunk', 'slow loris',
    'squirrel', 'turtle', 'walrus', 'wolf', 'wolverine', 'wombat',
]


def get_random_anonymous_animal():
    return random.choice(ANONYMOUS_ANIMALS)



# Imports from django.  # NOQA
from django.core.management.base import BaseCommand, CommandError  # NOQA


# Imports from editorial-staff.
from editorial_staff.loaders.slack import load_staff_from_slack


class Command(BaseCommand):
    help = 'Makes bylines for all current, full-privileged Slack users'

    def handle(self, *args, **options):
        created_bylines = load_staff_from_slack()

        self.stdout.write(self.style.SUCCESS(
            'Successfully created {} bylines:'.format(len(created_bylines))
        ))

        for byline in created_bylines:
            self.stdout.write(self.style.SUCCESS(
                '    - {} ({})'.format(
                    byline.render_full_name(),
                    byline.email,
                )
            ))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-21 07:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('editorial_staff', '0003_auto_20170821_1239'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staffer',
            name='image_url',
            field=models.URLField(blank=True, help_text='Usually auto-populated from Slack.', null=True),
        ),
    ]
